#include "graph.hpp"
void Graph::draw()
{
	_boxprint();
	drawPoints();
	wrefresh(win);
}
void Graph::drawPoints()
{
	int col=0;
	for(auto it = points.begin(); it != points.end(); ++it)
	{
		if(it->y < x/5) col = hi;
		if(it->y > x/2) col = low;
		wattron(win, COLOR_PAIR(col) | A_BOLD);
		mvwaddch(win,it->y,it->x,'#');
		wattroff(win,COLOR_PAIR(col) | A_BOLD);
		col = mid;	
	}
}
// Do rysowania wykresu stosuje algorytm bresenhama
void Graph::DrawLine(const int x1, const int y1, const int x2, const int y2)
{ 
     // zmienne pomocnicze
     int d, dx, dy, ai, bi, xi, yi;
     int x = x1, y = y1;
     // ustalenie kierunku rysowania
     if (x1 < x2)
     { 
         xi = 1;
         dx = x2 - x1;
     } 
     else
     { 
         xi = -1;
         dx = x1 - x2;
     }
     // ustalenie kierunku rysowania
     if (y1 < y2)
     { 
         yi = 1;
         dy = y2 - y1;
     } 
     else
     { 
         yi = -1;
         dy = y1 - y2;
     }
     // pierwszy piksel
     points.push_back(Point(x, y));
     // oś wiodąca OX
     if (dx > dy)
     {
         ai = (dy - dx) * 2;
         bi = dy * 2;
         d = bi - dx;
         // pętla po kolejnych x
         while (x != x2)
         { 
             // test współczynnika
             if (d >= 0)
             { 
                 x += xi;
                 y += yi;
                 d += ai;
             } 
             else
             {
                 d += bi;
                 x += xi;
             }
             points.push_back(Point(x, y));
         }
     } 
     // oś wiodąca OY
     else
     { 
         ai = ( dx - dy ) * 2;
         bi = dx * 2;
         d = bi - dy;
         // pętla po kolejnych y
         while (y != y2)
         { 
             // test współczynnika
             if (d >= 0)
             { 
                 x += xi;
                 y += yi;
                 d += ai;
             }
             else
             {
                 d += bi;
                 y += yi;
             }
             points.push_back(Point(x, y));
         }
     }
 }
void Graph::DrawCircle(const int _x0, const int _y0, const int radius)
{
    int x = 0;
    int y = radius;
    int delta = 2 - 2 * radius;
    int error = 0;

    while(y >= 0)
    {
		points.push_back(Point(_x0 + x,_y0 + y));
		points.push_back(Point(_x0 - x,_y0 + y));
		points.push_back(Point(_x0 + x,_y0 - y));
		points.push_back(Point(_x0 - x,_y0 - y));
        error = 2 * (delta + y) - 1;
        if(delta < 0 && error <= 0) {
            ++x;
            delta += 2 * x + 1;
            continue;
        }
        error = 2 * (delta - x) - 1;
        if(delta > 0 && error > 0) {
            --y;
            delta += 1 - 2 * y;
            continue;
        }
        ++x;
        delta += 2 * (x - y);
        --y;
    }
}
void Graph::DrawEllipse(const int _x0, const int _y0, const int width, const int height)
{
    int a2 = width * width;
    int b2 = height * height;
    int fa2 = 4 * a2, fb2 = 4 * b2;
    int x, y, sigma;
    for (x = 0, y = height, sigma = 2*b2+a2*(1-2*height); b2*x <= a2*y; x++)
    {
		points.push_back(Point(_x0 + x,_y0 + y));
		points.push_back(Point(_x0 - x,_y0 + y));
		points.push_back(Point(_x0 + x,_y0 - y));
		points.push_back(Point(_x0 - x,_y0 - y));
        if (sigma >= 0)
        {
            sigma += fa2 * (1 - y);
            y--;
        }
        sigma += b2 * ((4 * x) + 6);
    }
    for (x = width, y = 0, sigma = 2*a2+b2*(1-2*width); a2*y <= b2*x; y++)
    {
		points.push_back(Point(_x0 + x,_y0 + y));
		points.push_back(Point(_x0 - x,_y0 + y));
		points.push_back(Point(_x0 + x,_y0 - y));
		points.push_back(Point(_x0 - x,_y0 - y));
        if (sigma >= 0)
        {
            sigma += fb2 * (1 - x);
            x--;
        }
        sigma += a2 * ((4 * y) + 6);
    }
}
